"use client";

import Image from "next/image";
import { useForm } from "react-hook-form";
import { useRouter } from "next/navigation";
import { useState, useTransition } from "react";
import { zodResolver } from "@hookform/resolvers/zod";
import styles from "@/styles/pages/login.module.css";
import Input from "@/components/Input";
import Button from "@/components/Button";
import Loader from "@/components/Loader";
import { FormError } from "@/components/FormError";
import { LoginSchema } from "@/types/login";
import { loginSchema } from "@/schemas/login";
import { login } from "@/app/_actions/login";
import { SetTokenLocalStorage } from "@/utils/set-token";
import Logo from "@/app/assets/logo.png";

export default function Login() {
  const [isPending, starTransition] = useTransition();
  const [errorMessage, setErrorMessage] = useState("");

  const router = useRouter();

  const {
    register,
    handleSubmit,
    formState: { errors },
    trigger,
  } = useForm<LoginSchema>({
    resolver: zodResolver(loginSchema),
    mode: "onChange",
    defaultValues: {
      username: "",
      password: "",
    },
  });

  const submit = ({ username, password }: LoginSchema) => {
    setErrorMessage("");

    starTransition(async () => {
      const isValid = await trigger();

      if (!isValid) return;

      login({
        username,
        password,
      }).then((response) => {
        if (response.message) {
          setErrorMessage(response.message);
          return;
        }
        const { token } = response.data;

        SetTokenLocalStorage(token);

        router.push("/");
      });
    });
  };

  return (
    <div className={styles.container}>
      <div className={styles.headerContainer}>
        <Image
          src={Logo}
          alt="Logo"
          width={460}
          height={60}
          className={styles.logo}
        />
        <span className="font-inter">Portal Administrativo</span>
      </div>

      <form className={styles.formContainer} onSubmit={handleSubmit(submit)}>
        <Input
          register={register("username")}
          label="Email"
          placeholder="Insira seu email"
          type="email"
          errorMessage={errors.username?.message}
        />
        <Input
          register={register("password")}
          label="Senha"
          placeholder="Insira sua senha"
          type="password"
          errorMessage={errors.password?.message}
        />
        <div className={styles.button}>
          <FormError message={errorMessage} />
          <Button type="submit" variant="default">
            {isPending ? <Loader loaderType="login" /> : "Entrar"}
          </Button>
        </div>
      </form>
    </div>
  );
}
