import React, { useState, useEffect } from "react";
import styles from "@/styles/components/listItem.module.css";
import { Trash, Edit, MessageCircle } from "@geist-ui/icons";
import DeleteModal from "@/components/DeleteModal";
import { Post } from "@/types/post";
import { deletePost } from "@/app/_actions/posts";
import { fetchPostById } from "../app/_actions/posts";
import RegisterContentModal from "./RegisterContentModal";
import FeedbackVisualization from "@/components/FeedbackVisualization";
import { Feedback } from "@/types/post";


interface ListItemTableProps {
  posts: Post[];
}

const ListItemTable: React.FC<ListItemTableProps> = ({
  posts: initialItems,
}) => {
  const [items, setItems] = useState<Post[]>(initialItems);
  const [deletingItemId, setDeletingItemId] = useState<string | null>(null);
  const [isModalOpen, setIsModalOpen] = useState(false);
  const [postData, setPostData] = useState<Post | null>(null);
  const [feedbackModalOpen, setFeedbackModalOpen] = useState(false);
  const [selectedFeedbacks, setSelectedFeedbacks] = useState<Feedback[]>([]);

  useEffect(() => {
    setItems(initialItems);
  }, [initialItems]);

  const handleDeleteClick = (id: string) => {
    setDeletingItemId(id);
  };

  const handleConfirmDelete = async () => {
    if (deletingItemId !== null) {
      setItems(items.filter((item) => item.id !== deletingItemId));
      await deletePost(deletingItemId);
      setDeletingItemId(null);
    }
  };

  const handleCancelDelete = () => {
    setDeletingItemId(null);
  };

  const openModalForEdit = async (postId: string) => {
    try {
      const post = await fetchPostById(postId);
      setPostData(post);
      setIsModalOpen(true);
    } catch (error) {
      console.error("Erro ao buscar post:", error);
    }
  };

  const openFeedbackModal = async (postId: string) => {
    try {
      const post = await fetchPostById(postId);
      setSelectedFeedbacks(post.feedbacks);
      setFeedbackModalOpen(true);
    } catch (error) {
      console.error("Erro ao buscar feedbacks:", error);
    }
  };

  const closeModal = () => {
    setIsModalOpen(false);
    setPostData(null);
  };

  const closeFeedbackModal = () => {
    setFeedbackModalOpen(false);
    setSelectedFeedbacks([]);
  };

  return (
    <div>
      <table className={styles.listTable}>
        <thead>
          <tr>
            <th>Título</th>
            <th>Categoria</th>
            <th>Data de criação</th>
            <th></th>
          </tr>
        </thead>
        <tbody>
          {items.map((item) => (
            <tr key={item.id}>
              <td>{item.title.length > 30 ? `${item.title.slice(0, 30)}...` : item.title}</td>
              <td>{item.category.join(", ")}</td>
              <td>{"3/11/2024"}</td>
              <td className={styles["actions-column"]}>
                {item.feedbacks && (
                  <button
                    className={styles["icon-btn"]}
                    onClick={() => openFeedbackModal(item.id)}
                  >
                    <MessageCircle />
                  </button>
                )}
                <button
                  className={styles["icon-btn"]}
                  onClick={() => handleDeleteClick(item.id)}
                >
                  <Trash />
                </button>
                <button
                  className={styles["icon-btn"]}
                  onClick={() => openModalForEdit(item.id)}
                >
                  <Edit />
                </button>
              </td>
            </tr>
          ))}
        </tbody>
      </table>

      {deletingItemId !== null && (
      <DeleteModal
        isOpen={true}
        onClose={handleCancelDelete}
        onConfirm={handleConfirmDelete}
        title="Você tem certeza que deseja excluir este item?"
        description="Essa ação não pode ser desfeita. O item será permanentemente removido."
      />
      )}


    <RegisterContentModal
      isOpen={isModalOpen}
      onClose={closeModal}
      refreshPosts={() => {}}
      postId={postData?.id}
    />
    
      {feedbackModalOpen && (
        <FeedbackVisualization
          feedbacks={selectedFeedbacks}
          onClose={closeFeedbackModal}
        />
      )}
    </div>
  );
};

export default ListItemTable;
