"use client";

import { toast } from "sonner";
import { useEffect, useState } from "react";

import { Post } from "@/interfaces/post";

import { fetchPosts } from "@/app/_actions/posts/fetch-posts";

import Loader from "@/components/Loader";
import Button from "@/components/Button";
import SearchInput from "@/components/SearchInput";
import TablePost from "@/components/TableContent";
import CardContainer from "@/components/CardContainer";
import TableSkeleton from "@/components/TableSkeleton";
import RegisterContentModal from "@/components/RegisterContentModal";

import useDebounce from "@/hooks/useDebounce";

import { PlusSquare } from "@geist-ui/icons";

import styles from "@styles/components/register.content.module.css";

export default function RegisterContent({ refreshKey, onRefresh }: { refreshKey: number; onRefresh: () => void }) {
  const [isOpen, setIsOpen] = useState(false);
  const [posts, setPosts] = useState<Post[]>([]);
  const [searchPost, setSearchPost] = useState("");
  const [isLoading, setIsLoading] = useState(true);
  const [isSearchLoading, setIsSearchLoading] = useState(false);
  const [filteredPosts, setFilteredPosts] = useState<Post[]>([]);
  const [editingPostId, setEditingPostId] = useState<string | null>(null);

  const debounceSearch = useDebounce(searchPost);

  const refreshPosts = () => {
    setIsLoading(true);

    fetchPosts({})
      .then((response) => {
        if (response.message) toast.error(response.message);

        setPosts(response.data);
        setFilteredPosts(response.data);
      })
      .catch(() => {
        toast.error("Erro ao buscar os conteúdos.");
      })
      .finally(() => {
        setIsLoading(false);
      });
  };

  useEffect(() => {
    refreshPosts();
  }, [refreshKey]);

  useEffect(() => {
    if (!debounceSearch) {
      setFilteredPosts(posts);
      setIsSearchLoading(false);
      return;
    }

    setIsSearchLoading(true);

    const filtered = posts.filter((post) =>
      post.title.toLowerCase().includes(debounceSearch.toLowerCase())
    );

    setFilteredPosts(filtered);

    setTimeout(() => {
      setIsSearchLoading(false);
    }, 600);
  }, [debounceSearch, posts]);

  const handleOpenModal = (postId: string | null) => {
    setEditingPostId(postId);
    setIsOpen(true);
  };

  const handleCloseModal = () => {
    setIsOpen(false);
  };

  return (
    <CardContainer>
      <div className={styles.headerContainer}>
        <h1 className={styles.titleHeader}>Conteúdo</h1>
        <div className={styles.searchInputContainer}>
          <SearchInput
            placeholder="Pesquise o conteúdo"
            onChange={setSearchPost}
          />
          <Button onClick={() => handleOpenModal(null)} variant="default" className={styles.headerButton}>
            <PlusSquare className={styles.buttonIcon} size={24} />
            Novo conteúdo
          </Button>
        </div>
      </div>
      <div className={styles.tableContainer}>
        {isLoading && <TableSkeleton />}
        {!isLoading && isSearchLoading && <Loader loaderType="component" />}
        {!isLoading && !isSearchLoading && filteredPosts.length > 0 && (
          <TablePost
            refreshPosts={refreshPosts} 
            posts={filteredPosts} 
            onEditPoint={handleOpenModal}
          />
        )}
        {!isLoading && !isSearchLoading && filteredPosts.length === 0 && debounceSearch && (
          <p className={styles.notFoundMessage}>Post não encontrado</p>
        )}
      </div>
      <RegisterContentModal 
        isOpen={isOpen} 
        onClose={handleCloseModal} 
        refreshPosts={() => {
          refreshPosts();
          onRefresh();
        }}
        postId={editingPostId}
      />
    </CardContainer>
  );
}
