/* eslint-disable @next/next/no-img-element */
/* eslint-disable react-hooks/exhaustive-deps */
'use client';

import { toast } from 'sonner';
import { useForm } from 'react-hook-form';
import { zodResolver } from '@hookform/resolvers/zod';
import { IoMdClose, IoMdCloudUpload } from "react-icons/io";
import React, { ChangeEvent, useEffect, useState, useTransition } from 'react';

import { selectOptions } from '@/data';

import Input from '@/components/Input';
import Modal from "@/components/Modal";
import Button from '@/components/Button';
import Loader from '@/components/Loader';
import Select from '@/components/Select';

import { RegisterSchema } from '@/types/content';

import { PostRequest } from '@/interfaces/post';
import { PointOfInterestRequest } from '@/interfaces/point-of-interest';

import { registerContentSchema } from '@/schemas/content';

import { createPost } from '@/app/_actions/posts/create-post';
import { getCoordinates } from '@/app/_actions/get-coordinates';

import styles from '@styles/components/register.content.modal.module.css';
import { fetchPost } from '@/app/_actions/posts/fetch-post';
import { getAddressFromCoordinates } from '@/app/_actions/get-address-from-coordinates';
import { updatePost } from '@/app/_actions/posts/update-post';

interface ContentModalProps {
  isOpen: boolean;
  onClose: () => void;
  refreshPosts: () => void;
  postId?: string | null;
}

export default function RegisterContentModal({
  isOpen,
  onClose,
  refreshPosts,
  postId
}: ContentModalProps) {
  const [fileBase64, setFileBase64] = useState<string | null>(null);
  const [isPending, startTransition] = useTransition();
  const [selectedOptions, setSelectedOptions] = useState<string[]>([]);
  const [isLoading, setIsLoading] = useState(false);

  const {
    register,
    handleSubmit,
    setValue,
    trigger,
    reset,
    formState: { errors },
  } = useForm<RegisterSchema>({
    resolver: zodResolver(registerContentSchema),
    mode: 'onChange',
  });

  useEffect(() => {
    if (isOpen && postId) {
      setIsLoading(true);
    
      fetchPost(postId)
        .then(async (response) => {
          if (response.message) return toast.error(response.message);
    
          const { title, description, category, pointOfInterest, image } = response.data;
    
          setValue('title', title);
          setValue('description', description || '');
          setValue('categories', category || []);
          setSelectedOptions(category || []);
    
          if (image) {
            const fullBase64 = image.startsWith('data:image') 
              ? image 
              : `data:image/jpeg;base64,${image}`;
            setFileBase64(fullBase64);
            setValue('image', fullBase64);
          }
    
          if (pointOfInterest) {
            const { titulo, descricao, latitude, longitude, address, zipCode, state } = pointOfInterest;
    
            setValue('addressData.title', titulo);
            setValue('addressData.description', descricao || '');
    
            if (latitude && longitude) {
              const addressResponse = await getAddressFromCoordinates({ lat: latitude, lng: longitude });
    
              if (addressResponse.message) return toast.error(addressResponse.message);
    
              setValue('addressData.address', addressResponse.address);
              setValue('addressData.zipCode', addressResponse.cep);
              setValue('addressData.state', addressResponse.state);

              return;
            }

            setValue('addressData.address', address || '');
            setValue('addressData.zipCode', zipCode || '');
            setValue('addressData.state', state || '');
          }
        })
        .catch(() => {
          toast.error("Erro ao buscar os dados do post.");
        })
        .finally(() => {
          setIsLoading(false);
        });
    } else if (!isOpen) {
      reset();
      setFileBase64(null);
      setSelectedOptions([]);
    }
  }, [postId, isOpen]);
  

  const onSubmit = async ({
    title,
    description,
    categories,
    addressData,
    image,
  }: RegisterSchema) => {
    startTransition(async () => {
      const isValid = await trigger();

      if (!isValid) return;

      let payload: PostRequest = {
        title,
        description,
        category: categories,
        image: image || null,
        isActive: true,
      };

      if (addressData?.address?.trim() && addressData?.zipCode?.trim() && addressData?.state?.trim()) {
        try {
          const response = await getCoordinates({
            address: addressData.address,
            cep: addressData.zipCode,
            state: addressData.state,
          });

          if (response.message) {
            toast.error(response.message);
            return;
          }

          const { lat, lng } = response.data.results[0].geometry.location;

          const pointOfInterest: PointOfInterestRequest = {
            titulo: String(title),
            descricao: String(description),
            latitude: String(lat),
            longitude: String(lng),
          };

          payload = {
            ...payload,
            pointOfInterest,
            category: [...categories, "PontosDeInteresse"],
          };
        } catch (error) {
          toast.error("Erro ao buscar as coordenadas.");
          return;
        }
      }

      try {
        if (postId) {
          const response = await updatePost({ post_id: postId, payload });

          if (response.message) {
            toast.error(response.message);
            return;
          }
          
          toast.success('Post atualizado com sucesso!');
        } 

        const response = await createPost(payload);

        if (response.message) {
          toast.error(response.message);
          return;
        }

        toast.success('Post cadastrado com sucesso!');
      
        setTimeout(() => {
          onClose();
          refreshPosts();
          reset();
        }, 1000);
      } catch {
        toast.error("Erro ao salvar os dados do post.");
      }
    });
  };

  const handleSelectChange = (values: string[]) => {
    setSelectedOptions(values);
    setValue('categories', values);
  };

  const handleInputImageChange = (e: ChangeEvent<HTMLInputElement>) => {
    if (e.target.files && e.target.files.length > 0) {
      const file = e.target.files[0];

      const reader = new FileReader();
      reader.onloadend = () => {
        if (reader.result) {
          const fullBase64 = reader.result.toString();
          const base64Data = fullBase64.replace(/^data:image\/\w+;base64,/, '');

          setFileBase64(fullBase64);
          setValue('image', base64Data);
        } else {
          console.error("Erro ao converter a imagem para Base64.");
        }
      };

      reader.readAsDataURL(file);
    }
  };

  const handleRemoveImage = () => {
    setFileBase64(null);
    setValue('image', '');
  };

  if (!isOpen) return null;

  return (
    <Modal
      isOpen={isOpen}
      onClose={() => {
        onClose();
        reset();
        setFileBase64(null);
        setSelectedOptions([]);
      }}
      title={postId ? 'Editar Post' : 'Cadastrar Post'}
    >
      {isLoading && <Loader loaderType='component' />}
      {!isLoading && (
        <form onSubmit={handleSubmit(onSubmit)} className={styles.form}>
          <div className={styles.inputGroup}>
            <label htmlFor="title" className={styles.label}>Título:</label>
            <div className={styles.inputWrapper}>
              <Input
                id="title"
                placeholder="Título do conteúdo"
                register={register('title')}
                errorMessage={errors.title?.message}
              />
            </div>
          </div>

          <div className={styles.inputGroup}>
            <label htmlFor="description" className={styles.label}>Descrição:</label>
            <div className={styles.inputWrapper}>
              <textarea
                id="description"
                placeholder="Adicione a descrição do conteúdo"
                {...register('description')}
                className={styles.textarea}
              />
              {errors.description?.message && (
                <span className={styles.errorMessage}>
                  {errors.description.message}
                </span>
              )}
            </div>
          </div>

          <div className={styles.inputGroup}>
            <label htmlFor="category" className={styles.label}>Categoria:</label>
            <Select 
              onChange={handleSelectChange} 
              options={selectOptions} 
              defaultOptions={selectOptions.filter(option => selectedOptions.includes(option.value))} 
              errorMessage={errors.categories?.message && "Selecione pelo menos uma opção"}
            />
          </div>

          <div className={styles.inputGroup}>
            <label className={styles.pointOfInterestLabel}>
              Ponto de Interesse:
            </label>
            <div className={styles.inputWrapper}>
              <Input
                placeholder="Endereço"
                register={register('addressData.address')}
                errorMessage={errors.addressData?.address?.message}
              />
              <Input
                placeholder="Título"
                register={register('addressData.title')}
                errorMessage={errors.addressData?.title?.message}
              />
              <Input
                placeholder="Descrição"
                register={register('addressData.description')}
                errorMessage={errors.addressData?.description?.message}
              />
              <div className={styles.formRow}>
                <Input
                  placeholder="CEP"
                  register={register('addressData.zipCode')}
                  errorMessage={errors.addressData?.zipCode?.message}
                  mask='cep'
                />
                <Input
                  placeholder="Estado"
                  register={register('addressData.state')}
                  errorMessage={errors.addressData?.state?.message}
                />
              </div>
            </div>
            <div className={styles.inputGroup}>
              <label className={styles.pointOfInterestLabel}>
                Imagem:
              </label>
              <div className={styles.inputWrapper}>
                <input
                  type="file"
                  accept="image/*"
                  id="fileUpload"
                  className={styles.fileInput}
                  onChange={handleInputImageChange}
                />
                <label htmlFor="fileUpload" className={styles.uploadButton}>
                  <IoMdCloudUpload className={styles.uploadIcon} />
                  Upload
                </label>
                {fileBase64 && (
                  <div className={styles.imagePreviewContainer}>
                    <button
                      className={styles.removeImageButton}
                      onClick={handleRemoveImage}
                      type="button"
                    >
                      <IoMdClose />
                    </button>
                    <img
                      src={fileBase64}
                      alt="Prévia da imagem"
                      className={styles.previewImage}
                    />
                  </div>
                )}
              </div>
            </div>
          </div>
          <div className={styles.modalFooter}>
            <Button
              type="submit"
              variant="default"
              className={styles.submitButton}
            >
              {isPending ? <Loader loaderType="login" /> : postId ? 'Salvar Alterações' : 'Cadastrar'}
            </Button>
          </div>
        </form>
      )}
    </Modal>
  );
}
