"use client";

import { toast } from "sonner";
import { useEffect, useState } from "react";

import { Post } from "@/interfaces/post";
import { PointOfInterest } from "@/interfaces/point-of-interest";

import { fetchPosts } from "@/app/_actions/posts/fetch-posts";

import Loader from "@/components/Loader";
import Button from "@/components/Button";
import SearchInput from "@/components/SearchInput";
import CardContainer from "@/components/CardContainer";
import TableSkeleton from "@/components/TableSkeleton";
import TablePointsOfInterest from "@/components/TablePointOfInteres";
import RegisterPointOfInterestModal from "@/components/RegisterPointOfInterestModal";

import useDebounce from "@/hooks/useDebounce";

import { PlusSquare } from "@geist-ui/icons";

import styles from "@styles/components/register.content.module.css";

export default function RegisterPointOfInterest({ refreshKey, onRefresh }: { refreshKey: number; onRefresh: () => void }) {
  const [isOpen, setIsOpen] = useState(false);
  const [isLoading, setIsLoading] = useState(true);
  const [searchPoint, setSearchPoint] = useState("");
  const [points, setPoints] = useState<PointOfInterest[]>([]);
  const [isSearchLoading, setIsSearchLoading] = useState(false);
  const [editingPostId, setEditingPostId] = useState<string | null>(null);
  const [editingPointId, setEditingPointId] = useState<string | null>(null);
  const [filteredPoints, setFilteredPoints] = useState<PointOfInterest[]>([]);

  const debounceSearch = useDebounce(searchPoint);

  const refreshPoints = () => {
    setIsLoading(true);
  
    fetchPosts({
      category_name: "PontosDeInteresse",
    })
      .then((response) => {
        if (response.message) {
          toast.error(response.message);
          return;
        }
  
        const allPoints: PointOfInterest[] = response.data
          .map((post: Post) => {
            if (
              post.pointOfInterest &&
              post.category.length === 1 &&
              post.category.includes("PontosDeInteresse")
            ) {
              return {
                ...post.pointOfInterest,
                postId: post.id,
              };
            }
            return null;
          })
          .filter(
            (point: PointOfInterest): point is PointOfInterest =>
              point !== null && point.id !== null && point.titulo !== undefined
          );
  
        setPoints(allPoints);
        setFilteredPoints(allPoints);
      })
      .catch(() => {
        toast.error("Erro ao buscar os pontos de interesse.");
      })
      .finally(() => {
        setIsLoading(false);
      });
  };

  useEffect(() => {
    refreshPoints();
  }, [refreshKey]);

  useEffect(() => {
    if (!debounceSearch) {
      setFilteredPoints(points);
      setIsSearchLoading(false);
      return;
    }

    setIsSearchLoading(true);

    const filtered = points.filter((point) =>
      point.titulo.toLowerCase().includes(debounceSearch.toLowerCase())
    );

    setFilteredPoints(filtered);

    setTimeout(() => {
      setIsSearchLoading(false);
    }, 600);
  }, [debounceSearch, points]);

  const handleOpenModal = (pointId: string | null, postId: string | null) => {
    setEditingPointId(pointId); 
    setEditingPostId(postId);
    setIsOpen(true);
  };

  const handleCloseModal = () => {
    setEditingPointId(null);
    setIsOpen(false);
  };

  return (
    <CardContainer>
      <div className={styles.headerContainer}>
        <h1 className={styles.titleHeader}>Pontos de Interesse</h1>
        <div className={styles.searchInputContainer}>
          <SearchInput
            placeholder="Pesquise pelo título do ponto"
            onChange={setSearchPoint}
          />
          <Button
            onClick={() => handleOpenModal(null, null)}
            variant="default"
            className={styles.headerButton}
          >
            <PlusSquare className={styles.buttonIcon} size={24} />
            Novo Ponto
          </Button>
        </div>
      </div>
      <div className={styles.tableContainer}>
        {isLoading && <TableSkeleton />}
        {!isLoading && isSearchLoading && <Loader loaderType="component" />}
        {!isLoading && !isSearchLoading && filteredPoints.length > 0 && (
          <TablePointsOfInterest
            refreshPoints={refreshPoints}
            points={filteredPoints}
            onEditPoint={handleOpenModal}
          />
        )}
        {!isLoading && !isSearchLoading && filteredPoints.length === 0 && debounceSearch && (
          <p className={styles.notFoundMessage}>Nenhum ponto encontrado</p>
        )}
      </div>
      <RegisterPointOfInterestModal
        isOpen={isOpen}
        onClose={handleCloseModal}
        refreshPoints={() => {
          refreshPoints();
          onRefresh();
        }}
        pointId={editingPointId}
        postId={editingPostId}
      />
    </CardContainer>
  );
}
