/* eslint-disable react-hooks/exhaustive-deps */
"use client";

import { toast } from "sonner";
import { useTransition } from "react";
import { useForm } from "react-hook-form";
import { useEffect, useState } from "react";

import Modal from "@/components/Modal";
import Input from "@/components/Input";
import Loader from "@/components/Loader";
import Button from "@/components/Button";

import { zodResolver } from "@hookform/resolvers/zod";

import { PointOfInterestSchema } from "@/types/point-of-interes";
import { pointOfInteresSchema } from "@/schemas/point-of-interest";

import { PointOfInterestRequest } from "@/interfaces/point-of-interest";

import { fetchPost } from "@/app/_actions/posts/fetch-post";
import { getCoordinates } from "@/app/_actions/get-coordinates";
import { createPoint } from "@/app/_actions/points/create-point";
import { updatePoint } from "@/app/_actions/points/update-point";
import { getAddressFromCoordinates } from "@/app/_actions/get-address-from-coordinates";

import styles from "@styles/components/register.point.of.interes.modal.module.css";

interface RegisterPointOfInterestModalProps {
  isOpen: boolean;
  onClose: () => void;
  refreshPoints: () => void;
  pointId?: string | null;
  postId?: string | null
}

export default function RegisterPointOfInterestModal({ 
  isOpen, 
  onClose, 
  refreshPoints,
  pointId,
  postId,
}: RegisterPointOfInterestModalProps) {
  const [isLoading, setIsLoading] = useState(false);
  const [isPending, startTransition] = useTransition();

  const {
    register,
    handleSubmit,
    trigger,
    reset,
    setValue,
    formState: { errors },
  } = useForm<PointOfInterestSchema>({
    resolver: zodResolver(pointOfInteresSchema),
    mode: "onChange",
  });

  useEffect(() => {
    if (pointId && postId) {
      setIsLoading(true);
  
      fetchPost(postId)
        .then((response) => {
          if (response.message) return toast.error(response.message);
  
          const { pointOfInterest } = response.data;
  
          if (!pointOfInterest) return;
  
          const { titulo, descricao, latitude, longitude } = pointOfInterest;
  
          setValue("title", titulo);
          setValue("description", descricao || "");
  
          if (latitude && longitude) {
            getAddressFromCoordinates({ lat: latitude, lng: longitude })
              .then((response) => {
                if (response.message) return toast.error(response.message);
  
                const { address, cep, state } = response;

                setValue("address", address);
                setValue("zipCode", cep);
                setValue("state", state);
              })
              .catch(() => {
                toast.error("Erro ao buscar o endereço pelas coordenadas.");
              })
              .finally(() => {
                setIsLoading(false);
              });
          } else {
            setIsLoading(false);
          }
        })
        .catch(() => {
          toast.error("Erro desconhecido.");
          setIsLoading(false);
        });
    } else {
      reset();
    }
  }, [pointId, postId]);

  const onSubmit = ({ 
    address,
    description,
    state,
    title,
    zipCode,
  }: PointOfInterestSchema) => {
    startTransition(async () => {
      const isValid = await trigger();

      if (!isValid) return;
  
      if (pointId && postId) {
        getCoordinates({
          address,
          cep: zipCode,
          state,
        })
          .then((response) => {
            if (response.message) {
              toast.error(response.message);
              return;
            }
  
            const { lat, lng } = response.data.results[0].geometry.location;
  
            const payload: PointOfInterestRequest = {
              titulo: title,
              descricao: description,
              latitude: lat,
              longitude: lng,
            };
  
            updatePoint({
              post_id: postId,
              payload,
            })
              .then((response) => {
                if (response.message) {
                  toast.error(response.message);
                  return;
                }
  
                toast.success("Ponto de interesse atualizado com sucesso!");
  
                setTimeout(() => {
                  onClose();
                  refreshPoints();
                  reset();
                }, 1000);
              })
              .catch(() => {
                toast.error("Erro ao atualizar o ponto de interesse.");
              });
          })
          .catch(() => {
            toast.error("Erro ao buscar as novas coordenadas.");
          });
  
        return;
      }
  
      getCoordinates({
        address,
        cep: zipCode,
        state,
      })
        .then((response) => {
          if (response.message) return toast.error(response.message);
  
          const { lat, lng } = response.data.results[0].geometry.location;
  
          const payload: PointOfInterestRequest = {
            titulo: title,
            descricao: description,
            latitude: lat,
            longitude: lng,
          };
  
          createPoint(payload)
            .then((response) => {
              if (response.message) {
                toast.error(response.message);
                return;
              }
  
              toast.success("Ponto de interesse cadastrado com sucesso!");
  
              setTimeout(() => {
                onClose();
                refreshPoints();
                reset();
              }, 1000);
            });
        })
        .catch(() => {
          toast.error("Erro ao buscar as coordenadas.");
        });
    });
  };  

  return (
    <Modal
      title={pointId ? "Editar Ponto de Interesse" : "Cadastrar Ponto de Interesse"}
      isOpen={isOpen}
      onClose={() => {
        onClose();
        reset();
      }}
    >
      {isLoading && <Loader loaderType="component" />}
      {!isLoading && (
        <form onSubmit={handleSubmit(onSubmit)} className={styles.form}>
          <Input 
            register={register("address")}
            placeholder="Endereço"
            errorMessage={errors.address?.message}
          />
          <Input 
            register={register("title")}
            placeholder="Título"
            errorMessage={errors.title?.message}
          />
          <Input 
            register={register("description")}
            placeholder="Descrição"
            errorMessage={errors.description?.message}
          />
          <Input 
            register={register("zipCode")}
            placeholder="CEP"
            errorMessage={errors.zipCode?.message}
            mask="cep"
          />
          <Input 
            register={register("state")}
            placeholder="Estado"
            errorMessage={errors.state?.message}
          />
          <div className={styles.modalFooter}>
            <Button
              type="submit"
              variant="default"
              className={styles.submitButton}
            >
              {isPending ? <Loader loaderType="login" /> : pointId && postId ? "Salvar Alterações" : "Cadastrar"}
            </Button>
          </div>
        </form>
      )}
    </Modal>
  );
}
