import React, { useState, useEffect, useRef } from 'react';
import { FiChevronDown } from 'react-icons/fi';
import styles from '@styles/components/select.module.css';

interface Option {
  value: string;
  label: string;
}

export interface SelectProps {
  options: Option[];
  placeholder?: string;
  label?: string;
  onChange: (values: string[]) => void;
  defaultOptions?: Option[];
  errorMessage?: string;
}

export default function Select({
  options,
  label,
  onChange,
  defaultOptions = [],
  errorMessage
}: SelectProps) {
  const [isOpen, setIsOpen] = useState(false);
  const [selectedOptions, setSelectedOptions] = useState<Option[]>(defaultOptions);
  const selectRef = useRef<HTMLDivElement>(null);

  useEffect(() => {
    if (defaultOptions) {
      setSelectedOptions(defaultOptions);
    }
  }, [defaultOptions]);

  const handleSelect = (option: Option) => {
    const alreadySelected = selectedOptions.some(selected => selected.value === option.value);

    if (alreadySelected) {
      const newSelections = selectedOptions.filter(selected => selected.value !== option.value);
      setSelectedOptions(newSelections);
      onChange(newSelections.map(option => option.value));
    } else {
      const newSelections = [...selectedOptions, option];
      setSelectedOptions(newSelections);
      onChange(newSelections.map(option => option.value));
    }
  };

  useEffect(() => {
    const handleClickOutside = (event: MouseEvent) => {
      if (selectRef.current && !selectRef.current.contains(event.target as Node)) {
        setIsOpen(false);
      }
    };

    document.addEventListener('mousedown', handleClickOutside);
    return () => {
      document.removeEventListener('mousedown', handleClickOutside);
    };
  }, []);

  return (
    <div ref={selectRef} className={styles.selectWrapper}>
      {label && <label className={styles.label}>{label}</label>}
      <div
        onClick={() => setIsOpen(!isOpen)}
        className={`${styles.select} ${isOpen ? styles.open : ''}`}
      >
        <span>
          {selectedOptions.length > 0
            ? selectedOptions.map(option => option.value).join(', ')
            : 'Selecione uma ou mais opções'}
        </span>
        <FiChevronDown
          className={`${styles.icon} ${isOpen ? styles.rotate : ''}`}
        />
      </div>

      {isOpen && (
        <ul className={styles.options}>
          {options.map((option) => (
            <li
              key={option.value}
              onClick={() => handleSelect(option)}
              className={`${styles.option} ${
                selectedOptions.some(selected => selected.value === option.value) ? styles.selected : ''
              }`}
            >
              {option.value}
            </li>
          ))}
        </ul>
      )}
      {errorMessage && (
        <span className={styles.errorMessage}>{errorMessage}</span>
      )}
    </div>
  );
}
