'use client'

import { toast } from 'sonner'
import React, { useState } from 'react'
import { BiMessageSquare, BiPencil, BiTrash } from 'react-icons/bi'

import { Post } from '@/interfaces/post'

import { deletePost } from '@/app/_actions/posts/delete-post'

import styles from '@styles/components/table.contained.module.css'
import { formatDate } from '@/utils/formatDate';
import FeedBackModal from '@/components/FeedBackModal'
import DeleteModal from '@/components/DeleteModal'

interface TablePostProps {
  posts: Post[]
  refreshPosts: () => void;
  onEditPoint: (postId: string) => void;
}

export default function TablePost({ posts, refreshPosts, onEditPoint }: TablePostProps) {
  const [postId, setPostId] = useState<string | null>(null);
  const [showModal, setShowModal] = useState({
    showDeleteModal: false,
    showFeedbacksModal: false,
  })

  const { showDeleteModal, showFeedbacksModal } = showModal

  const handleShowModal = (modalKey: keyof typeof showModal, post_id?: string) => {
    setShowModal((prevState) => ({
      ...prevState,
      [modalKey]: !prevState[modalKey],
    }))

    if (post_id) setPostId(post_id);
  }

  const handleDeletePost = () => {
    deletePost(postId as string)
      .then((response) => {
        if (response.message) return toast.error(response.message);

        toast.success("Conteúdo deletado com sucesso!");

        setTimeout(() => {
          refreshPosts();
          handleShowModal('showDeleteModal');
        })
      })
      .catch(() => {
        toast.error("Erro desconhecido.")
      })
  }

  return (
    <div className={styles.tableContainer}>
      <table className={styles.table}>
        <thead>
          <tr>
            <th>Título</th>
            <th>Categoria</th>
            <th>Data de criação</th>
            <th className={styles.actionsHeader}></th>
          </tr>
        </thead>
        <tbody>
          {posts.map((post) => (
            <tr key={post.id}>
              <td className={styles.cell}>{post.title}</td>
              <td className={styles.cell}>  
                {post.category.join(', ')}
              </td>
              <td className={styles.cell}>{formatDate(post.data_criacao)}</td>
              <td className={styles.actions}>
                <button onClick={() => handleShowModal('showFeedbacksModal', post.id)} className={styles.actionButton}>
                  <BiMessageSquare size={20} />
                </button>
                <button onClick={() => handleShowModal('showDeleteModal', post.id)} className={styles.actionButton}>
                  <BiTrash size={20} />
                </button>
                <button onClick={() => onEditPoint(post.id)} className={styles.actionButton}>
                  <BiPencil size={20} />
                </button>
              </td>
            </tr>
          ))}
        </tbody>
      </table>
      <DeleteModal 
        isOpen={showDeleteModal}
        onClose={() => {
          handleShowModal('showDeleteModal');
          setPostId(null);
        }}
        title="Você tem certeza que deseja excluir o conteúdo?"
        description="Essa ação não pode ser desfeita. O conteúdo será permanentemente deletado e não irá ser exibido para os usuários."
        onConfirm={handleDeletePost}
      />
      {showFeedbacksModal && (
        <FeedBackModal
          isOpen={showFeedbacksModal}
          onClose={() => {
            handleShowModal('showFeedbacksModal');
            setPostId(null);
          }}
          post_id={postId as string}
        />
      )}
    </div>
  )
}