/* eslint-disable @typescript-eslint/no-explicit-any */
/* eslint-disable import/no-anonymous-default-export */
import axios, { AxiosInstance, AxiosRequestConfig } from "axios";

interface RequestProps {
  url: string;
  data?: unknown;
  headers?: Record<string, string>;
}

class ApiService {
  private axiosInstance: AxiosInstance;

  constructor() {
    this.axiosInstance = axios.create({
      baseURL: `${process.env.NEXT_PUBLIC_API_URL}`,
      headers: { "Content-Type": "application/json" },
    });
    this.initializeInterceptor();
  }

  private initializeInterceptor() {
    this.axiosInstance.interceptors.request.use(
      (config) => {
        if (typeof window !== "undefined") {
          const token = localStorage.getItem("token");

          if (token) {
            config.headers["Authorization"] = `Bearer ${token}`;
          }
        }

        return config;
      },
      (error) => Promise.reject(error)
    );
  }

  private async request({
    url,
    data,
    headers,
    method,
  }: RequestProps & { method: string }) {
    try {
      const config: AxiosRequestConfig = {
        url,
        method,
        data,
        headers,
      };

      const response = await this.axiosInstance.request(config);

      return {
        data: response.data,
      };
    } catch (error) {
      if (axios.isAxiosError(error)) {
        const apiErrorMessage =
          error.response?.data?.message ||
          error.response?.data?.error ||
          JSON.stringify(error.response?.data) ||
          "Erro desconhecido";

        return {
          message: apiErrorMessage,
          status: error.response?.status || 500,
        };
      } else {
        return {
          message: `Erro desconhecido ${error}`,
        };
      }
    }
  }

  public async post({ url, data, headers }: RequestProps) {
    return this.request({ url, data, headers, method: "post" });
  }

  public async get({ url, headers }: RequestProps) {
    return this.request({ url, headers, method: "get" });
  }

  public async patch({ url, data, headers }: RequestProps) {
    return this.request({ url, data, headers, method: "patch" });
  }

  public async delete({ url, headers }: RequestProps) {
    return this.request({ url, headers, method: "delete" });
  }

  public async put({ url, data, headers }: RequestProps) {
    return this.request({ url, data, headers, method: "put" });
  }
}

export default new ApiService();